/*
 * Copyright 2016 - 2022 Gary Talent (gary@drinkingtea.net). All rights reserved.
 */

#pragma once

#include <ox/model/def.hpp>

#include <nostalgia/core/gfx.hpp>
#include <nostalgia/geo/bounds.hpp>
#include <nostalgia/geo/vec.hpp>
#include <nostalgia/glutils/glutils.hpp>
#include <nostalgia/studio/studio.hpp>

#include "tilesheeteditormodel.hpp"
#include "tilesheetpixelgrid.hpp"
#include "tilesheetpixels.hpp"

namespace nostalgia::core {

class TileSheetEditor {

	private:
		TileSheetEditorModel m_model;
		TileSheetGrid m_pixelGridDrawer;
		TileSheetPixels m_pixelsDrawer;
		bool m_updated = false;
		float m_pixelSizeMod = 1;
		geo::Vec2 m_scrollOffset;

	public:
		TileSheetEditor(Context *ctx, const ox::String &path);

		~TileSheetEditor() = default;

		void cut();

		void copy();

		void paste();

		void draw() noexcept;

		void clickPixel(const geo::Vec2 &paneSize, const geo::Vec2 &clickPos) noexcept;

		void scrollV(const geo::Vec2 &paneSz, float wheel, bool zoomMod) noexcept;

		void scrollH(const geo::Vec2 &paneSz, float wheel) noexcept;

		void resize(const geo::Vec2 &sz) noexcept;

		[[nodiscard]]
		constexpr const NostalgiaGraphic &img() const noexcept;

		[[nodiscard]]
		constexpr const NostalgiaPalette &pal() const noexcept;

		[[nodiscard]]
		bool updated() const noexcept;

		void ackUpdate() noexcept;

	protected:
		void saveItem();

	private:
		void setPalette();

		void saveState();

		void restoreState();

		[[nodiscard]]
		ox::String paletteName(const ox::String &palettePath) const;

		[[nodiscard]]
		ox::String palettePath(const ox::String &palettePath) const;

	// slots
	public:
		ox::Error colorSelected() noexcept;

		ox::Error setColorTable() noexcept;

};

constexpr const NostalgiaGraphic &TileSheetEditor::img() const noexcept {
	return m_model.img();
}

constexpr const NostalgiaPalette &TileSheetEditor::pal() const noexcept {
	return m_model.pal();
}

}
